#!/bin/bash

regexRules=()
whiteLists=()
blackLists=()
declare -A processTypes # Map 형태의 자료구조


NPM_PATH=/usr/whatap/whatap-npmd/whatap_npmd
TAGRULE_PATH=/usr/whatap/whatap-npmd/tagRule.yaml
TAGRULE_BACKUP_PATH=/usr/whatap/whatap-npmd/tagRule.backup
INSTALL_PATH=/usr/whatap/whatap-npmd/
DEFAULT_TAGRULE_PATH=/usr/whatap/whatap-npmd/defaultTagRule.yaml

detail=$1

if [ "$EUID" != "0" ]; then
	echo "This script must be run as root or with sudo" >&2
	exit
fi

function printProcessInfo {
    printf "%-20s %-15s %-15s\n" "Process Name" "Process Type" "App Name"
    if test -f "$NPM_PATH"; then
        string=`$NPM_PATH -s`
        IFS="|" read -ra elements <<< "$string"
        for element in "${elements[@]}"; do
            IFS="&" read -ra values <<< "$element"
            printf "%-20s %-15s %-15s\n" "${values[0]}" "${values[1]}" "${values[2]}"
        done
    else
        echo "Whatap NPM Agent Install이 선행되어야 합니다."
        exit
    fi
}

function setting_tagrule {
    clear
    while true; do
        echo "==============================================="
        echo "1. Process 정보 조회"
        echo "2. 입력된 TagRule 조회"
        echo "3. PrecessRegEx 옵션 입력 (Process Name 간소화 옵션)"
        echo "4. WhiteList 옵션 입력"
        echo "5. BlackLIst 옵션 입력"
        echo "6. ProcessType 옵션 입력"
        echo "7. AppName 옵션 입력"
        echo "8. UntagOption 옵션 입력"
        echo "9. yaml 저장 후 조회(기존 파일 백업)"
        echo "0. 종료"

        read -p "숫자를 입력하세요: " choice

        clear

        case $choice in
            1)
                printProcessInfo
                ;;
            2)
                echo "ProcessRegEx 옵션 목록"
                for regexRule in "${regexRules[@]}"; do
                    echo "    - $regexRule"
                done

                echo "WhiteList 옵션 목록"
                for whiteList in "${whiteLists[@]}"; do
                    echo "    - $whiteList"
                done

                echo "BlackList 옵션 목록"
                for blackList in "${blackLists[@]}"; do
                    echo "    - $blackList"
                done

                echo "ProcessType 옵션 목록"
                for key in "${!processTypes[@]}"; do
                    values="${processTypes[$key]}"
                    IFS=" " read -ra names <<< "$values"
                    echo "    $key:"
                    for name in "${names[@]}"; do
                        echo "        - $name"
                    done
                done
                ;;
            3)
                regexRules=()
                read -p "RegExRule의 개수를 입력하세요: " numRegex
                for ((i=1; i<=numRegex; i++)); do
                    read -p "RegExRule $i 입력: " regexRule
                    echo "입력받은 RegExRule $i: $regexRule"
                    regexRules+=("$regexRule")
                done
                ;;
            4)
                whiteLists=()
                read -p "WhiteList의 개수를 입력하세요: " numWhiteList
                for ((i=1; i<=numWhiteList; i++)); do
                    read -p "WhiteList $i 입력: " whiteList
                    echo "입력받은 WhiteList $i: $whiteList"
                    whiteLists+=("$whiteList")
                done
                ;;
            5)
                blackLists=()
                read -p "BlackList의 개수를 입력하세요: " numBlackList
                for ((i=1; i<=numBlackList; i++)); do
                    read -p "BlackList $i 입력: " blackList
                    echo "입력받은 BlackList $i: $blackList"
                    blackLists+=("$blackList")
                done
                ;;
            6)
                read -p "ProcessType의 개수를 입력하세요: " numProcessType
                for ((i=1; i<=numProcessType; i++)); do
                    read -p "Process Type 입력: " processType
                    names=()
                    while true; do
                        read -p "$processType 에 해당하는 processName을 입력하세요 (입력을 종료하려면 'exit' 입력): " processName
                        if [ "$processName" == "exit" ]; then
                            break
                        fi
                        names+=("$processName")
                    done
                    processTypes["$processType"]="${names[@]}"
                done

                ;;
            9)

                if test -f "$TAGRULE_PATH"; then
                    mv $TAGRULE_PATH $TAGRULE_BACKUP_PATH
                fi 

                echo "processRegEx:" > $TAGRULE_PATH
                for regexRule in "${regexRules[@]}"; do
                    echo "    - $regexRule" >> $TAGRULE_PATH
                done

                echo "processWhiteList:" >> $TAGRULE_PATH
                for whiteList in "${whiteLists[@]}"; do
                    echo "    - $whiteList" >> $TAGRULE_PATH
                done

                echo "processBlackList:" >> $TAGRULE_PATH
                for blackList in "${blackLists[@]}"; do
                    echo "    - $blackList" >> $TAGRULE_PATH
                done

                echo "processType:" >> $TAGRULE_PATH
                for key in "${!processTypes[@]}"; do
                    value="${processTypes[$key]}"
                    IFS=" " read -ra names <<< "$values"
                    echo "    $key:" >> $TAGRULE_PATH
                    for name in "${names[@]}"; do
                        echo "        - $name" >> $TAGRULE_PATH
                    done
                done

                printProcessInfo
                ;;
            0)
                exit
                ;;
        esac
    done
}

function setting_tagrule_simple {
    if test -f "$TAGRULE_PATH"; then
        echo "Old files are backed up. ($TAGRULE_PATH -> $TAGRULE_BACKUP_PATH)" 
        mv $TAGRULE_PATH $TAGRULE_BACKUP_PATH
        exit
    fi 

    regexRules=()
    output=$(netstat -tp)

    # PID 목록 추출하여 배열에 저장
    pids=($(echo "$output" | awk '{print $NF}' | grep -oE '[0-9]+' | sort -u))

    echo "================================================================================================================"
    echo ""
    echo "TCP/UDP 소켓과 연관된 프로세스들 중 Alias가 필요한 항목들을 추천합니다."
    echo "정규 표현식의 형태로 입력 가능하며, 정규 표현식에 매칭된 문자열을 Alias로 사용합니다."
    echo "각 프로세스 별 패턴이 겹치지 않아야하며 너무 짧으면 의도하지 않은 프로세스가 같은 패턴으로 매칭될 수 있습니다."
    echo ""

    if test -f "$NPM_PATH"; then
        string=`$NPM_PATH -p`
        IFS="|" read -ra elements <<< "$string"
        for element in "${elements[@]}"; do
            IFS="&" read -ra values <<< "$element"
            if [[ " ${pids[@]} " =~ " ${values[0]} " ]]; then
                read -p "${values[1]} 매칭에 사용할 패턴을 입력하세요 : " regex

                regexRules+=("$regex")
            fi
        done

        echo "processRegEx:" > $TAGRULE_PATH
        for regexRule in "${regexRules[@]}"; do
            echo "    - $regexRule" >> $TAGRULE_PATH
        done
    else
        echo "Whatap NPM Agent Install이 선행되어야 합니다."
        exit
    fi

    # 배열 출력
    #for pid in "${pids[@]}"; do
    #    echo "$pid"
    #done
    cat $DEFAULT_TAGRULE_PATH >> $TAGRULE_PATH
}

function check_rule {
    echo ""
    echo "================================================================================================================"
    echo ""
    echo "연관 프로세스 리스트"
    echo "Tag 규칙을 변경하려면 $TAGRULE_PATH 파일을 확인하세요."

    output=$(netstat -tp)

    pids=($(echo "$output" | awk '{print $NF}' | grep -oE '[0-9]+' | sort -u))
    printf "%-20s %-15s %-15s\n" "Process Name" "Process Type" "App Name"
    if test -f "$NPM_PATH"; then
        string=`$NPM_PATH -s`
        IFS="|" read -ra elements <<< "$string"
        for element in "${elements[@]}"; do
            IFS="&" read -ra values <<< "$element"
            if [[ " ${pids[@]} " =~ " ${values[3]} " ]]; then
                printf "%-20s %-15s %-15s\n" "${values[0]}" "${values[1]}" "${values[2]}"
            fi

        done

    else
        echo "Whatap NPM Agent Install이 선행되어야 합니다."
        exit
    fi
}


if [ -n "$detail" ]; then 
    setting_tagrule
else
    setting_tagrule_simple
    check_rule
fi

